# Generated by Django 5.2 on 2025-05-25 12:04

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("bionty", "0051_alter_cellline__branch_code_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="cellline",
            old_name="_branch_code",
            new_name="branch",
        ),
        migrations.RenameField(
            model_name="cellmarker",
            old_name="_branch_code",
            new_name="branch",
        ),
        migrations.RenameField(
            model_name="celltype",
            old_name="_branch_code",
            new_name="branch",
        ),
        migrations.RenameField(
            model_name="developmentalstage",
            old_name="_branch_code",
            new_name="branch",
        ),
        migrations.RenameField(
            model_name="disease",
            old_name="_branch_code",
            new_name="branch",
        ),
        migrations.RenameField(
            model_name="ethnicity",
            old_name="_branch_code",
            new_name="branch",
        ),
        migrations.RenameField(
            model_name="experimentalfactor",
            old_name="_branch_code",
            new_name="branch",
        ),
        migrations.RenameField(
            model_name="gene",
            old_name="_branch_code",
            new_name="branch",
        ),
        migrations.RenameField(
            model_name="organism",
            old_name="_branch_code",
            new_name="branch",
        ),
        migrations.RenameField(
            model_name="pathway",
            old_name="_branch_code",
            new_name="branch",
        ),
        migrations.RenameField(
            model_name="phenotype",
            old_name="_branch_code",
            new_name="branch",
        ),
        migrations.RenameField(
            model_name="protein",
            old_name="_branch_code",
            new_name="branch",
        ),
        migrations.RenameField(
            model_name="source",
            old_name="_branch_code",
            new_name="branch",
        ),
        migrations.RenameField(
            model_name="tissue",
            old_name="_branch_code",
            new_name="branch",
        ),
    ]
