# Generated by Django 5.2 on 2025-06-03 09:46

import django.db.models.deletion
import lamindb.base.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("bionty", "0053_recordcellline_cellline_records_recordcellmarker_and_more"),
        ("lamindb", "0104_alter_branch_uid"),
    ]

    operations = [
        migrations.AlterField(
            model_name="cellline",
            name="branch",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_column="_branch_code",
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.branch",
            ),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="space",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.space",
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="branch",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_column="_branch_code",
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.branch",
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="space",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.space",
            ),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="branch",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_column="_branch_code",
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.branch",
            ),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="space",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.space",
            ),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="branch",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_column="_branch_code",
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.branch",
            ),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="space",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.space",
            ),
        ),
        migrations.AlterField(
            model_name="disease",
            name="branch",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_column="_branch_code",
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.branch",
            ),
        ),
        migrations.AlterField(
            model_name="disease",
            name="space",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.space",
            ),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="branch",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_column="_branch_code",
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.branch",
            ),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="space",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.space",
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="branch",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_column="_branch_code",
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.branch",
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="space",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.space",
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="branch",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_column="_branch_code",
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.branch",
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="space",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.space",
            ),
        ),
        migrations.AlterField(
            model_name="organism",
            name="branch",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_column="_branch_code",
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.branch",
            ),
        ),
        migrations.AlterField(
            model_name="organism",
            name="space",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.space",
            ),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="branch",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_column="_branch_code",
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.branch",
            ),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="space",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.space",
            ),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="branch",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_column="_branch_code",
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.branch",
            ),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="space",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.space",
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="branch",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_column="_branch_code",
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.branch",
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="space",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.space",
            ),
        ),
        migrations.AlterField(
            model_name="source",
            name="branch",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_column="_branch_code",
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.branch",
            ),
        ),
        migrations.AlterField(
            model_name="source",
            name="space",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.space",
            ),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="branch",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_column="_branch_code",
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.branch",
            ),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="space",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                db_default=1,
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.space",
            ),
        ),
    ]
