# Generated by Django 5.2 on 2025-01-27 07:22

import django.db.models.deletion
import lamindb.base.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("bionty", "0048_lamindbv1_part6"),
    ]

    operations = [
        migrations.AlterField(
            model_name="schemacellmarker",
            name="cellmarker",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_schema",
                to="bionty.cellmarker",
            ),
        ),
        migrations.AlterField(
            model_name="schemacellmarker",
            name="schema",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_cellmarker",
                to="lamindb.schema",
            ),
        ),
        migrations.AlterField(
            model_name="schemagene",
            name="gene",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_schema",
                to="bionty.gene",
            ),
        ),
        migrations.AlterField(
            model_name="schemagene",
            name="schema",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_gene",
                to="lamindb.schema",
            ),
        ),
        migrations.AlterField(
            model_name="schemapathway",
            name="pathway",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_schema",
                to="bionty.pathway",
            ),
        ),
        migrations.AlterField(
            model_name="schemapathway",
            name="schema",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_pathway",
                to="lamindb.schema",
            ),
        ),
        migrations.AlterField(
            model_name="schemaprotein",
            name="protein",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_schema",
                to="bionty.protein",
            ),
        ),
        migrations.AlterField(
            model_name="schemaprotein",
            name="schema",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_protein",
                to="lamindb.schema",
            ),
        ),
    ]
