# Generated by Django 5.2 on 2025-07-26 15:56

import lamindb.base.fields
from django.db import migrations

import bionty.uids


class Migration(migrations.Migration):
    dependencies = [
        ("bionty", "0056_alter_recordtissue_record"),
    ]

    operations = [
        migrations.AlterField(
            model_name="cellline",
            name="description",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="synonyms",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=bionty.uids.ontology,
                max_length=8,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="description",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="synonyms",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=bionty.uids.cellmarker,
                max_length=12,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="description",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="synonyms",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=bionty.uids.ontology,
                max_length=8,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="description",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="synonyms",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=bionty.uids.ontology,
                max_length=8,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="disease",
            name="description",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="disease",
            name="synonyms",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="disease",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=bionty.uids.ontology,
                max_length=8,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="description",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="synonyms",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=bionty.uids.ontology,
                max_length=8,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="description",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="synonyms",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=bionty.uids.ontology,
                max_length=8,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="description",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="ncbi_gene_ids",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="synonyms",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=bionty.uids.gene,
                max_length=12,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="organism",
            name="description",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="organism",
            name="synonyms",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="organism",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=bionty.uids.ontology,
                max_length=8,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="description",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="synonyms",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=bionty.uids.ontology,
                max_length=8,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="description",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="synonyms",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=bionty.uids.ontology,
                max_length=8,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="description",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="synonyms",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=bionty.uids.protein,
                max_length=12,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="description",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="synonyms",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=bionty.uids.ontology,
                max_length=8,
                unique=True,
            ),
        ),
    ]
