# Generated by Django 5.1 on 2024-07-25 13:59

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("bionty", "0029_alter_cellline_previous_runs_and_more"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="PublicSource",
            new_name="Source",
        ),
        migrations.RenameField(
            model_name="cellline",
            old_name="public_source",
            new_name="source",
        ),
        migrations.RenameField(
            model_name="cellmarker",
            old_name="public_source",
            new_name="source",
        ),
        migrations.RenameField(
            model_name="celltype",
            old_name="public_source",
            new_name="source",
        ),
        migrations.RenameField(
            model_name="developmentalstage",
            old_name="public_source",
            new_name="source",
        ),
        migrations.RenameField(
            model_name="disease",
            old_name="public_source",
            new_name="source",
        ),
        migrations.RenameField(
            model_name="ethnicity",
            old_name="public_source",
            new_name="source",
        ),
        migrations.RenameField(
            model_name="experimentalfactor",
            old_name="public_source",
            new_name="source",
        ),
        migrations.RenameField(
            model_name="gene",
            old_name="public_source",
            new_name="source",
        ),
        migrations.RenameField(
            model_name="organism",
            old_name="public_source",
            new_name="source",
        ),
        migrations.RenameField(
            model_name="pathway",
            old_name="public_source",
            new_name="source",
        ),
        migrations.RenameField(
            model_name="phenotype",
            old_name="public_source",
            new_name="source",
        ),
        migrations.RenameField(
            model_name="protein",
            old_name="public_source",
            new_name="source",
        ),
        migrations.RenameField(
            model_name="tissue",
            old_name="public_source",
            new_name="source",
        ),
        migrations.AddField(
            model_name="source",
            name="in_db",
            field=models.BooleanField(db_index=True, default=False),
        ),
        migrations.AddField(
            model_name="source",
            name="artifacts",
            field=models.ManyToManyField(
                related_name="reference_of_sources", to="lamindb.artifact"
            ),
        ),
        migrations.AddField(
            model_name="source",
            name="df",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="reference_of_source",
                to="lamindb.artifact",
            ),
        ),
        migrations.AddField(
            model_name="cellmarker",
            name="description",
            field=models.TextField(default=None, null=True),
        ),
        migrations.AddField(
            model_name="organism",
            name="description",
            field=models.TextField(default=None, null=True),
        ),
        migrations.AddField(
            model_name="organism",
            name="synonyms",
            field=models.TextField(default=None, null=True),
        ),
        migrations.AddField(
            model_name="protein",
            name="description",
            field=models.TextField(default=None, null=True),
        ),
        migrations.AlterField(
            model_name="protein",
            name="name",
            field=models.CharField(
                db_index=True, default=None, max_length=256, null=True
            ),
        ),
        migrations.RenameField(
            model_name="artifactcellline",
            old_name="cell_line_ref_is_name",
            new_name="label_ref_is_name",
        ),
        migrations.RenameField(
            model_name="artifactcellmarker",
            old_name="cell_marker_ref_is_name",
            new_name="label_ref_is_name",
        ),
        migrations.RenameField(
            model_name="artifactcelltype",
            old_name="cell_type_ref_is_name",
            new_name="label_ref_is_name",
        ),
        migrations.RenameField(
            model_name="artifactdevelopmentalstage",
            old_name="developmental_stage_ref_is_name",
            new_name="label_ref_is_name",
        ),
        migrations.RenameField(
            model_name="artifactdisease",
            old_name="disease_ref_is_name",
            new_name="label_ref_is_name",
        ),
        migrations.RenameField(
            model_name="artifactethnicity",
            old_name="ethnicity_ref_is_name",
            new_name="label_ref_is_name",
        ),
        migrations.RenameField(
            model_name="artifactexperimentalfactor",
            old_name="experimental_factor",
            new_name="experimentalfactor",
        ),
        migrations.RenameField(
            model_name="artifactexperimentalfactor",
            old_name="experimental_factor_ref_is_name",
            new_name="label_ref_is_name",
        ),
        migrations.RenameField(
            model_name="artifactorganism",
            old_name="organism_ref_is_name",
            new_name="label_ref_is_name",
        ),
        migrations.RenameField(
            model_name="artifactpathway",
            old_name="pathway_ref_is_name",
            new_name="label_ref_is_name",
        ),
        migrations.RenameField(
            model_name="artifactphenotype",
            old_name="phenotype_ref_is_name",
            new_name="label_ref_is_name",
        ),
        migrations.RenameField(
            model_name="artifactprotein",
            old_name="protein_ref_is_name",
            new_name="label_ref_is_name",
        ),
        migrations.RenameField(
            model_name="artifacttissue",
            old_name="tissue_ref_is_name",
            new_name="label_ref_is_name",
        ),
    ]
