# Generated by Django 5.1.12 on 2025-10-01 14:39

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("bionty", "0060_alter_cellline_is_locked_alter_cellmarker_is_locked_and_more"),
    ]

    operations = []  # type: ignore
