# Generated by Django 5.0.7 on 2024-07-30 13:23

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("bionty", "0032_rename_source_name_source_description_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="artifactcellline",
            name="artifact",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_cell_line",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellline",
            name="cellline",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.cellline",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellline",
            name="feature",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactcellline",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellmarker",
            name="artifact",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_cell_marker",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellmarker",
            name="cellmarker",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.cellmarker",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellmarker",
            name="feature",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactcellmarker",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcelltype",
            name="artifact",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_cell_type",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcelltype",
            name="celltype",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.celltype",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcelltype",
            name="feature",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactcelltype",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdevelopmentalstage",
            name="artifact",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_developmental_stage",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdevelopmentalstage",
            name="developmentalstage",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.developmentalstage",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdevelopmentalstage",
            name="feature",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactdevelopmentalstage",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdisease",
            name="artifact",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_disease",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdisease",
            name="disease",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.disease",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdisease",
            name="feature",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactdisease",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactethnicity",
            name="artifact",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_ethnicity",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactethnicity",
            name="ethnicity",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.ethnicity",
            ),
        ),
        migrations.AlterField(
            model_name="artifactethnicity",
            name="feature",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactethnicity",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactexperimentalfactor",
            name="artifact",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_experimental_factor",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactexperimentalfactor",
            name="experimentalfactor",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.experimentalfactor",
            ),
        ),
        migrations.AlterField(
            model_name="artifactexperimentalfactor",
            name="feature",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactexperimentalfactor",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactgene",
            name="artifact",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_gene",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactgene",
            name="feature",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactgene",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactgene",
            name="gene",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.gene",
            ),
        ),
        migrations.AlterField(
            model_name="artifactorganism",
            name="artifact",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_organism",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactorganism",
            name="feature",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactorganism",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactorganism",
            name="organism",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.organism",
            ),
        ),
        migrations.AlterField(
            model_name="artifactpathway",
            name="artifact",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_pathway",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactpathway",
            name="feature",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactpathway",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactpathway",
            name="pathway",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.pathway",
            ),
        ),
        migrations.AlterField(
            model_name="artifactphenotype",
            name="artifact",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_phenotype",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactphenotype",
            name="feature",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactphenotype",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactphenotype",
            name="phenotype",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.phenotype",
            ),
        ),
        migrations.AlterField(
            model_name="artifactprotein",
            name="artifact",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_protein",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactprotein",
            name="feature",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactprotein",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactprotein",
            name="protein",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.protein",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttissue",
            name="artifact",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_tissue",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttissue",
            name="feature",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifacttissue",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttissue",
            name="tissue",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.tissue",
            ),
        ),
        migrations.RenameField(
            model_name="cellline",
            old_name="previous_runs",
            new_name="_previous_runs",
        ),
        migrations.RenameField(
            model_name="cellmarker",
            old_name="previous_runs",
            new_name="_previous_runs",
        ),
        migrations.RenameField(
            model_name="celltype",
            old_name="previous_runs",
            new_name="_previous_runs",
        ),
        migrations.RenameField(
            model_name="developmentalstage",
            old_name="previous_runs",
            new_name="_previous_runs",
        ),
        migrations.RenameField(
            model_name="disease",
            old_name="previous_runs",
            new_name="_previous_runs",
        ),
        migrations.RenameField(
            model_name="ethnicity",
            old_name="previous_runs",
            new_name="_previous_runs",
        ),
        migrations.RenameField(
            model_name="experimentalfactor",
            old_name="previous_runs",
            new_name="_previous_runs",
        ),
        migrations.RenameField(
            model_name="gene",
            old_name="previous_runs",
            new_name="_previous_runs",
        ),
        migrations.RenameField(
            model_name="organism",
            old_name="previous_runs",
            new_name="_previous_runs",
        ),
        migrations.RenameField(
            model_name="pathway",
            old_name="previous_runs",
            new_name="_previous_runs",
        ),
        migrations.RenameField(
            model_name="phenotype",
            old_name="previous_runs",
            new_name="_previous_runs",
        ),
        migrations.RenameField(
            model_name="protein",
            old_name="previous_runs",
            new_name="_previous_runs",
        ),
        migrations.RenameField(
            model_name="source",
            old_name="previous_runs",
            new_name="_previous_runs",
        ),
        migrations.RenameField(
            model_name="tissue",
            old_name="previous_runs",
            new_name="_previous_runs",
        ),
    ]
